// Code original de drdnar
// https://github.com/drdnar/Arduino-HID-Remote

#include "HIDRemote.h"

#define _ID_HIDRemote 0x04  // 

static const uint8_t _hidRemoteReportDescriptor[] PROGMEM =
{
// http://www2.ece.rochester.edu/~parihar/pres/Report_MultiMedia-KB.pdf
   0x05, 0x0c,          // USAGE_PAGE (Consumer Devices)
   0x09, 0x01,          // USAGE (Consumer Control)
   0xa1, 0x01,          // COLLECTION (Application)
   0x85, _ID_HIDRemote, // REPORT_ID (4)
   0x15, 0x00,          // LOGICAL_MINIMUM (0)
   0x25, 0x01,          // LOGICAL_MAXIMUM (1)
   0x75, 0x01,          // REPORT_SIZE (1)
   0x95, 0x10,          // REPORT_COUNT (16)
   0x09, 0xe2,          // USAGE (Mute) 0x01
   0x09, 0xe9,          // USAGE (Volume Up) 0x02
   0x09, 0xea,          // USAGE (Volume Down) 0x03
   0x09, 0xcd,          // USAGE (Play/Pause) 0x04
   0x09, 0xb7,          // USAGE (Stop) 0x05
   0x09, 0xb6,          // USAGE (Scan Previous Track) 0x06
   0x09, 0xb5,          // USAGE (Scan Next Track) 0x07
   0x0a, 0x8a, 0x01,    // USAGE (Mail) 0x08
   0x0a, 0x92, 0x01,    // USAGE (Calculator) 0x09
   0x0a, 0x21, 0x02,    // USAGE (www search) 0x0a
   0x0a, 0x23, 0x02,    // USAGE (www home) 0x0b
   0x0a, 0x2a, 0x02,    // USAGE (www favorites) 0x0c
   0x0a, 0x27, 0x02,    // USAGE (www refresh) 0x0d
   0x0a, 0x26, 0x02,    // USAGE (www stop) 0x0e
   0x0a, 0x25, 0x02,    // USAGE (www forward) 0x0f
   0x0a, 0x24, 0x02,    // USAGE (www back) 0x10
   0x81, 0x62, // INPUT (Data,Var,Abs,NPrf,Null)
   0xc0,
};

HIDRemote_::HIDRemote_(void)
{
    static HIDSubDescriptor node(_hidRemoteReportDescriptor, sizeof(_hidRemoteReportDescriptor));
    HID().AppendDescriptor(&node);
}
 
void HIDRemote_::begin(void) 
{
    report.keys[0] = 0;
    report.keys[1] = 0;
}
 
void HIDRemote_::end(void) 
{
}

void HIDRemote_::sendReport(RemoteReport* r)
{
    HID().SendReport(_ID_HIDRemote, r, sizeof(RemoteReport));
    delay(50);
}

void HIDRemote_::sendReport()
{
    HID().SendReport(_ID_HIDRemote, &report, sizeof(RemoteReport));
    delay(50);
}

void HIDRemote_::execute(uint16_t key)
{
    press(key);
    release(key);
}

void HIDRemote_::press(uint16_t key)
{
    report.keys[0] |= (key & 0xFF);
    report.keys[1] |= (key >> 8);
    sendReport();
}

void HIDRemote_::release(uint16_t key)
{
    report.keys[0] &= ~(key & 0xFF);
    report.keys[1] &= ~(key >> 8);
    sendReport();
}

void HIDRemote_::releaseAll(void)
{
    report.keys[0] = 0;
    report.keys[1] = 0;
    sendReport();
}

HIDRemote_ Media;
